#!/usr/bin/perl

use strict;
use warnings;

my $retval=0;

map {&verify_dir($_)} @ARGV;

exit $retval;

sub verify_dir {
    my ($dir)=@_;
    die "$dir: not a directory!\n" unless -d $dir;
    die "$dir/fonts.dir not found\n" unless -e "$dir/fonts.dir";

    open FF, '<', "$dir/fonts.dir" or die "can't open $dir/fonts.dir: $!";
    <FF>;
    my %font;
    while (<FF>) {
	chomp;
	next if /^\!/ or /^\s*$/;
	s/^"[^"]+"\s+"?// || s/^\S+\s+"?// || die "unparseable line $_";
	s/"?\s*$//;
	$font{$_}=1 if $_;
	#print $_, "\n";
    }
    close FF;
    return unless -e "$dir/fonts.alias";

    my @FAIL;
    my @WARN;
    open FF, '<', "$dir/fonts.alias" or die "can't open $dir/fonts.alias: $!";
    while (<FF>) {
	chomp;
	next if /^\!/ or /^\s*$/;
	my $alias;
	if (s/^"([^"]+)"\s+"?//) {
	    $alias=$1;
	} elsif (s/^(\S+)\s+"?//) {
	    $alias=$1;
	} else {
	    die "unparseable line $_";
	}
	s/"?\s*$//;
	next unless $_;
	my $fontdest=$_;
	if ($fontdest!~/\*/) {
	    unless ($font{$fontdest}) {
		push @FAIL, "not found $fontdest for alias $alias\n";
		$retval=3;
	    }
	} else {
	    my $pattern=&glob2pat($fontdest);
	    unless (scalar grep {/$pattern/} keys (%font)) {
		push @WARN, "glob $fontdest not found for alias $alias\n";
		$retval=2;
	    }
	}
    }
    close FF;
    print STDERR "WARNING: $dir/fonts.alias:\n", @WARN if @WARN;
    print STDERR "FATAL: $dir/fonts.alias:\n", @FAIL if @FAIL;
}

sub glob2pat {
    my $globstr = shift;
    my %patmap = (
	'*' => '.*',
	'?' => '.',
	'[' => '[',
	']' => ']',
	'{' => '(?:',
	'}' => ')',
	',' => '|',
	);
    $globstr =~ s{(.)} { $patmap{$1} || "\Q$1" }ge;
    return '^' . $globstr . '$';
}
